﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	public class Border : Microsoft.Office.Interop.Excel.Border
	{
		public const int FAULT_WEIGHT = -1;
		public const int DEFAULT_WEIGHT = 2;
		private static ExcelColor DEFAULT_COLOR = new ExcelColor("#DADADA");

		public Microsoft.Office.Interop.Excel.XlBordersIndex Index { get; private set; }
		public bool IsChanged { get; private set; }

		protected Borders parent;
		protected Microsoft.Office.Interop.Excel.XlLineStyle lineStyle;
		internal ExcelColor color;

		public Border(Borders parent, Microsoft.Office.Interop.Excel.XlBordersIndex index)
		{
			this.Index = index;

			this.parent = parent;
			this.lineStyle = Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone;
			this.color = DEFAULT_COLOR;

			this.IsChanged = false;
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get
			{
				return parent.Application;
			}
		}

		/// <summary>
		/// Returns or sets the primary color of the object. Read/write.
		/// </summary>
		public dynamic Color
		{
			get
			{
				return color.IntValue;
			}
			set
			{
				SetColor(new ExcelColor(value));
				UpdateBorder();
			}
		}

		/// <summary>
		/// Returns or sets an int value that represents the color table index of the border.
		/// </summary>
		public dynamic ColorIndex
		{
			get
			{
				return ExcelColorPalette.GetColorIndex(color);
			}
			set
			{
				SetColor(ExcelColorPalette.GetByIndex(value));
				UpdateBorder();
			}
		}

		/// <summary>
		/// Returns or sets the line style for the border. Read/write XlLineStyle.
		/// </summary>
		public dynamic LineStyle
		{
			get
			{
				return lineStyle;
			}
			set
			{
				if (value is Microsoft.Office.Interop.Excel.XlLineStyle)
				{
					Microsoft.Office.Interop.Excel.XlLineStyle style = value;
					SetLineStyle(style);
					UpdateBorder();
				}
			}
		}

		/// <summary>
		/// Returns an int value that represents the weight of the border. Read-only.
		/// </summary>
		public dynamic Weight
		{
			get
			{
				return DEFAULT_WEIGHT;
			}
			set {}
		}

		#region service members

		protected void UpdateBorder()
		{
			bool? update = null;
			if (IsChanged) update = lineStyle != Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone;

			bool? top = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop) ? update : null;
			bool? left = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft) ? update : null;
			bool? bottom = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom) ? update : null;
			bool? right = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight) ? update : null;
			bool? vertical = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlInsideVertical) ? update : null;
			bool? horizontal = (Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlInsideHorizontal) ? update : null;

			parent.SetBorders(
				top,
				left,
				bottom,
				right,
				vertical,
				horizontal,
				color.HtmlValue,
				lineStyle.ToString()
			);
		}

		internal void SetColor(ExcelColor color)
		{
			this.color = color;
			if (lineStyle == Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone)
			{
				lineStyle = Microsoft.Office.Interop.Excel.XlLineStyle.xlContinuous;
			}
			IsChanged = true;
		}

		internal void SetLineStyle(Microsoft.Office.Interop.Excel.XlLineStyle style)
		{
			switch (style)
			{
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlContinuous:
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlDouble:
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlSlantDashDot:
				style = Microsoft.Office.Interop.Excel.XlLineStyle.xlContinuous;
				break;
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlDash:
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlDashDot:
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlDashDotDot:
				style = Microsoft.Office.Interop.Excel.XlLineStyle.xlDash;
				break;
			case Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone:
				color = DEFAULT_COLOR;
				break;
			}
			lineStyle = style;
			IsChanged = true;
		}

		#endregion

		#region unused members

		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic ThemeColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic TintAndShade
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}		

		#endregion
	}
}
