﻿using System.Linq;
using Google.GData.Spreadsheets;

namespace GSpreadCOM.Domain
{
	sealed class Document
	{
		private SpreadsheetsService spreadsheets;
		private SpreadsheetEntry spreadsheet;

		public DocumentsService Documents { get; private set; }
		public bool IsReadOnly { get { return spreadsheet.ReadOnly; } }
		public string Title { get; set; }
		public string Author { get; set; }
		public string Summary { get; set; }
		public string Id { get; private set; }
		public SheetsService Sheets { get; private set; }

		public Document(DocumentsService documents, SpreadsheetsService spreadsheets, SpreadsheetEntry spreadsheet)
		{
			this.spreadsheets = spreadsheets;
			this.spreadsheet = spreadsheet;
			Documents = documents;
			Title = spreadsheet.Title.Text;
			Author = spreadsheet.Authors.Count > 0 ? spreadsheet.Authors[0].Name : null;
			Summary = spreadsheet.Summary.Text;
			Sheets = new SheetsService(this, spreadsheets, spreadsheet);
			Id = DocumentsService.GetSpreadsheetId(spreadsheet);
		}

		public void Close()
		{
			Documents.Close(this);
		}
	}
}
