﻿using System;
using System.Collections.Generic;
using Google.GData.Client;
using Google.GData.Spreadsheets;
using Google.Apis.Drive.v2.Data;

namespace GSpreadCOM.Domain
{
	sealed class DocumentsService
	{
		private const string spreadsheetUrl = "https://spreadsheets.google.com/feeds/spreadsheets/private/full";

		public ApplicationService Application { get; private set; }
		public List<Document> Items { get; private set; }

		public DocumentsService(ApplicationService app)
		{
			Application = app;
			Items = new List<Document>();
		}

		public static string GetSpreadsheetId(SpreadsheetEntry spreadsheet)
		{
			string[] uriParts = spreadsheet.Id.AbsoluteUri.Split('/');
			return uriParts[uriParts.Length - 1];
		}

		public Document CreateNew(string title)
		{
			var file = new File()
			{
				Title = title,
				Description = string.Format("Created via {0} at {1}", Config.APP_NAME, DateTime.Now.ToString()),
				MimeType = "application/vnd.google-apps.spreadsheet"
			};

			File result = Application.Drive.Files.Insert(file).Execute();

			return new Document(this, Application.Spreadsheets, getById(result.Id));
		}

		public Document Open(string title)
		{
			var doc = new Document(this, Application.Spreadsheets, getByTitle(title));

			Items.Add(doc);

			return doc;
		}

		public void Close(Document document)
		{
			Items.Remove(document);
		}

		private SpreadsheetEntry getByNamedParameter(String name, String value)
		{
			var query = new SpreadsheetQuery(String.Format("{0}?{1}={2}", spreadsheetUrl, name, value));
			SpreadsheetFeed feed = Application.Spreadsheets.Query(query);

			if (feed.Entries.Count < 1)
			{
				throw new GDataRequestException(String.Format("Document \"{0}\" not found", value));
			}

			return (SpreadsheetEntry) feed.Entries[0];
		}

		private SpreadsheetEntry getByTitle(String title)
		{
			return getByNamedParameter("title", title);
		}

		private SpreadsheetEntry getById(String id)
		{
			return getByNamedParameter("id", id);
		}
	}
}
