﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	public class Interior : Microsoft.Office.Interop.Excel.Interior
	{
		public const string DEFAULT_COLOR = "white";

		protected Range range;

		public Interior(Range range)
		{
			this.range = range;
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return range.Application; }
		}

		/// <summary>
		/// Returns or sets the primary color of the object. Read/write.
		/// </summary>
		public dynamic Color
		{
			get
			{
				string htmlColor = GetBackgroundColor();
				if (htmlColor == null || htmlColor == "varies")
				{
					return ExcelColor.FAULT_VALUE;
				}
				return new ExcelColor(htmlColor).IntValue;
			}
			set
			{
				SetBackgroundColor(new ExcelColor(value).HtmlValue);
			}
		}

		/// <summary>
		/// Returns or sets an int value that represents the color table index of the interior.
		/// </summary>
		public dynamic ColorIndex
		{
			get
			{
				string htmlColor = GetBackgroundColor();
				if (htmlColor == null || htmlColor == "varies")
				{
					return ExcelColor.FAULT_VALUE;
				}
				return ExcelColorPalette.GetColorIndex(htmlColor);
			}
			set
			{
				SetBackgroundColor(ExcelColorPalette.GetByIndex(value).HtmlValue);
			}
		}

		#region service members

		protected string GetBackgroundColor()
		{
			return range.ScriptService.call("getBackgroundColor", range);
		}

		protected void SetBackgroundColor(string htmlColor) {
			range.ScriptService.call("setBackgroundColor", range, htmlColor);
		}

		#endregion

		#region unused members

		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic Gradient
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic InvertIfNegative
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic Pattern
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic PatternColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic PatternColorIndex
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic PatternThemeColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic PatternTintAndShade
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic ThemeColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic TintAndShade
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		#endregion
	}
}
