﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GSpreadCOM
{
	public class Style : Microsoft.Office.Interop.Excel.Style
	{
		protected const string LEFT = "left";
		protected const string CENTER = "center";
		protected const string RIGHT = "right";
		protected const string TOP = "top";
		protected const string MIDDLE = "middle";
		protected const string BOTTOM = "bottom";

		protected const Microsoft.Office.Interop.Excel.XlHAlign DEFAULT_HORIZONTAL_ALIGNMENT = Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignLeft;
		protected const Microsoft.Office.Interop.Excel.XlVAlign DEFAULT_VERTICAL_ALIGNMENT = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignBottom;
		protected const bool DEFAULT_WRAP_TEXT = false;
		protected const string DEFAULT_NUMBER_FORMAT = "0.###############";

		protected Range range;
		protected Borders borders;
		protected Font font;
		protected Interior interior;

		public Style(Range range)
		{
			this.range = range;
			this.borders = new Borders(range);
			this.interior = new Interior(range);
			this.font = new Font(range);
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return range.Application; }
		}

		/// <summary>
		/// Returns a Borders collection that represents the borders of a style or a range of cells (including a range defined as part of a conditional format).
		/// </summary>
		public Microsoft.Office.Interop.Excel.Borders Borders
		{
			get
			{
				return borders;
			}
		}

		/// <summary>
		/// Returns a Font object that represents the font of the specified object.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Font Font
		{
			get
			{
				return font;
			}
		}

		/// <summary>
		/// Returns an Interior object that represents the interior of the specified object.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Interior Interior
		{
			get
			{
				return interior;
			}
		}

		/// <summary>
		/// Returns or sets a XlHAlign value that represents the horizontal alignment for the specified object.
		/// </summary>
		public Microsoft.Office.Interop.Excel.XlHAlign HorizontalAlignment
		{
			get
			{
				string hAlign = range.ScriptService.call("getHorizontalAlignment", range);
				Microsoft.Office.Interop.Excel.XlHAlign horizontalAlignment = Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignGeneral;
				switch (hAlign)
				{
				case LEFT:
					horizontalAlignment = Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignLeft;
					break;
				case CENTER:
					horizontalAlignment = Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignCenter;
					break;
				case RIGHT:
					horizontalAlignment = Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignRight;
					break;
				}
				return horizontalAlignment;
			}
			set
			{
				string hAlign;
				switch (value)
				{
				case Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignCenterAcrossSelection:
				case Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignCenter:
					hAlign = CENTER;
					break;
				case Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignLeft:
					hAlign = LEFT;
					break;
				case Microsoft.Office.Interop.Excel.XlHAlign.xlHAlignRight:
					hAlign = RIGHT;
					break;
				default:
					return;
				}
				range.ScriptService.call("setHorizontalAlignment", range, hAlign);
			}
		}

		/// <summary>
		/// Returns or sets a String value that represents the format code for the object.
		/// </summary>
		public string NumberFormat
		{
			get
			{
				return range.ScriptService.call("getNumberFormat", range);
			}
			set
			{
				range.ScriptService.call("setNumberFormat", range, value);
			}
		}

		/// <summary>
		/// Returns or sets a XlVAlign value that represents the vertical alignment of the specified object.
		/// </summary>
		public Microsoft.Office.Interop.Excel.XlVAlign VerticalAlignment
		{
			get
			{
				string vAlign = range.ScriptService.call("getVerticalAlignment", range);
				Microsoft.Office.Interop.Excel.XlVAlign verticalAlignment = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignBottom;
				switch (vAlign)
				{
				case MIDDLE:
					verticalAlignment = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignCenter;
					break;
				case TOP:
					verticalAlignment = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignTop;
					break;
				}
				return verticalAlignment;
			}
			set
			{
				string vAlign;
				switch (value)
				{
				case Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignTop:
					vAlign = TOP;
					break;
				case Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignCenter:
					vAlign = MIDDLE;
					break;
				case Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignBottom:
					vAlign = BOTTOM;
					break;
				default:
					return;
				}
				range.ScriptService.call("setVerticalAlignment", range, vAlign);
			}
		}

		/// <summary>
		/// Returns or sets a Boolean value that indicates if text is wrapped in cells.
		/// </summary>
		public bool WrapText
		{
			get
			{
				bool wrap;
				string wrapStr = range.ScriptService.call("getWrap", range);
				return (Boolean.TryParse(wrapStr, out wrap) && wrap);
			}
			set
			{
				range.ScriptService.call("setWrap", range, value);
			}
		}

		#region service members

		public void Reset()
		{
			borders.LineStyle = Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone;

			interior.Color = GSpreadCOM.Interior.DEFAULT_COLOR;

			HorizontalAlignment = DEFAULT_HORIZONTAL_ALIGNMENT;
			VerticalAlignment = DEFAULT_VERTICAL_ALIGNMENT;
			WrapText = DEFAULT_WRAP_TEXT;
			NumberFormat = DEFAULT_NUMBER_FORMAT;

			font.Reset();
		}

		#endregion

		#region unused members

		public bool AddIndent
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool BuiltIn
		{
			get { throw new NotImplementedException(); }
		}

		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic Delete()
		{
			throw new NotImplementedException();
		}

		public bool FormulaHidden
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludeAlignment
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludeBorder
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludeFont
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludeNumber
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludePatterns
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool IncludeProtection
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public int IndentLevel
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool Locked
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic MergeCells
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public string Name
		{
			get { throw new NotImplementedException(); }
		}

		public string NameLocal
		{
			get { throw new NotImplementedException(); }
		}

		public string NumberFormatLocal
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.XlOrientation Orientation
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		public int ReadingOrder
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool ShrinkToFit
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public string Value
		{
			get { throw new NotImplementedException(); }
		}

		public string _Default
		{
			get { throw new NotImplementedException(); }
		}

		#endregion
	}
}
