﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[Guid("BF750B42-20B8-4805-AE2F-4D51FBD73165")]
	[ComVisible(true)]
	public class Workbooks : Microsoft.Office.Interop.Excel.Workbooks
	{
		internal Application Application { get; private set; }
		internal Workbook Active { get; set; }
		internal DocumentsService Documents { get; private set; }

		protected List<Workbook> workbooks;

		internal Workbooks(Application app)
		{
			Application = app;
			Documents = Application.Service.Documents;
			workbooks = new List<Workbook>();
		}

		#region properties

		/// <summary>
		/// Returns a single object from a collection.
		/// </summary>
		/// <param name="Index"></param>
		/// <returns></returns>
		[DispId(0)]
		public Microsoft.Office.Interop.Excel.Workbook this[object Index]
		{
			get
			{
				if (Index == null)
				{
					throw new ArgumentNullException("Index");
				}

				if (Index is String)
				{
					foreach (Workbook workbook in workbooks)
					{
						if (workbook.Name.Equals(Index as String))
						{
							return workbook;
						}
					}

					return __Open(Index as String);
				}
				else
				{
					return workbooks[(int) Index - 1];
				}
			}
		}

		/// <summary>
		/// Returns an int value that represents the number of objects in the collection.
		/// </summary>
		[DispId(118)]
		public int Count
		{
			get
			{
				return workbooks.Count;
			}
		}

		/// <summary>
		/// Returns a single object from a collection.
		/// </summary>
		/// <param name="Index"></param>
		/// <returns></returns>
		[DispId(170)]
		Microsoft.Office.Interop.Excel.Workbook Microsoft.Office.Interop.Excel.Workbooks.get_Item(object Index)
		{
			return this[Index];
		}

		#endregion

		#region methods

		/// <summary>
		/// Creates a new workbook. The new workbook becomes the active workbook.
		/// </summary>
		/// <param name="Template">String, filename</param>
		/// <returns>Workbook</returns>
		[DispId(181)]
		[LCIDConversion(1)]
		public Microsoft.Office.Interop.Excel.Workbook Add([Optional]object Template)
		{
		    string title = "New Spreadsheet";
		    if (Template is String)
		    {
		        title = Template as String;
		    }

		    var doc = Documents.CreateNew(title);

		    var workbook = new Workbook(this, doc);
		    workbooks.Add(workbook);
		    Active = workbook;

		    return workbook;
		}

		internal void Close(Workbook workbook)
		{
			if (workbook != null)
			{
				Documents.Close(workbook.Document);

				workbooks.Remove(workbook);

				if (workbook == Active) {
					Active = workbooks.LastOrDefault();
				}
			}
		}

		/// <summary>
		/// Closes the object.
		/// </summary>
		[LCIDConversion(0)]
		[DispId(277)]
		public void Close()
		{
			Documents.Items.Clear();
			workbooks.Clear();
			Active = null;
		}

		/// <summary>
		/// Opens a workbook.
		/// </summary>
		/// <param name="Filename"></param>
		/// <param name="UpdateLinks">ignored</param>
		/// <param name="ReadOnly">ignored</param>
		/// <param name="Format">ignored</param>
		/// <param name="Password">ignored</param>
		/// <param name="WriteResPassword">ignored</param>
		/// <param name="IgnoreReadOnlyRecommended">ignored</param>
		/// <param name="Origin">ignored</param>
		/// <param name="Delimiter">ignored</param>
		/// <param name="Editable">ignored</param>
		/// <param name="Notify">ignored</param>
		/// <param name="Converter">ignored</param>
		/// <param name="AddToMru">ignored</param>
		/// <param name="Local">ignored</param>
		/// <param name="CorruptLoad">ignored</param>
		/// <returns>Workbook</returns>
		[LCIDConversion(15)]
		[DispId(1923)]
		public Microsoft.Office.Interop.Excel.Workbook Open(
			string Filename,
			[Optional]object UpdateLinks,
			[Optional]object ReadOnly,
			[Optional]object Format,
			[Optional]object Password,
			[Optional]object WriteResPassword,
			[Optional]object IgnoreReadOnlyRecommended,
			[Optional]object Origin,
			[Optional]object Delimiter,
			[Optional]object Editable,
			[Optional]object Notify,
			[Optional]object Converter,
			[Optional]object AddToMru,
			[Optional]object Local,
			[Optional]object CorruptLoad)
		{
			return __Open(Filename, true);
		}

		[TypeLibFunc(1088)]
		[DispId(682)]
		[LCIDConversion(13)]
		public Microsoft.Office.Interop.Excel.Workbook _Open(string Filename, object UpdateLinks, object ReadOnly, object Format, object Password, object WriteResPassword, object IgnoreReadOnlyRecommended, object Origin, object Delimiter, object Editable, object Notify, object Converter, object AddToMru)
		{
			return Open(Filename, UpdateLinks, ReadOnly, Format, Password, WriteResPassword,
				IgnoreReadOnlyRecommended, Origin, Delimiter, Editable, Notify, Converter, AddToMru,
				Local: true, CorruptLoad: false);
		}

		protected Microsoft.Office.Interop.Excel.Workbook __Open(String filename, bool setActive = false)
		{
			var doc = Documents.Open(filename);

			var workbook = new Workbook(this, doc);
			workbooks.Add(workbook);
			if (setActive)
			{
				Active = workbook;
			}

			return workbook;
		}

		#endregion

		#region unused members

		[DispId(148)]
		Microsoft.Office.Interop.Excel.Application Microsoft.Office.Interop.Excel.Workbooks.Application { get { return Application; } }

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator { get { throw new NotImplementedException(); } }

		[DispId(683)]
		[LCIDConversion(14)]
		[TypeLibFunc(1088)]
		public void __OpenText(string Filename, object Origin, object StartRow, object DataType, Microsoft.Office.Interop.Excel.XlTextQualifier TextQualifier, object ConsecutiveDelimiter, object Tab, object Semicolon, object Comma, object Space, object Other, object OtherChar, object FieldInfo, object TextVisualLayout)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(18)]
		[DispId(1924)]
		public void _OpenText(string Filename, object Origin, object StartRow, object DataType, Microsoft.Office.Interop.Excel.XlTextQualifier TextQualifier, object ConsecutiveDelimiter, object Tab, object Semicolon, object Comma, object Space, object Other, object OtherChar, object FieldInfo, object TextVisualLayout, object DecimalSeparator, object ThousandsSeparator)
		{
			throw new NotImplementedException();
		}

		[DispId(2071)]
		[TypeLibFunc(1088)]
		public Microsoft.Office.Interop.Excel.Workbook _OpenXML(string Filename, object Stylesheets)
		{
			throw new NotImplementedException();
		}

		[DispId(2070)]
		public bool CanCheckOut(string Filename)
		{
			throw new NotImplementedException();
		}

		[DispId(2069)]
		public void CheckOut(string Filename)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(1024)]
		[DispId(-4)]
		public IEnumerator GetEnumerator()
		{
		    return workbooks.GetEnumerator();
		}

		[DispId(2067)]
		public Microsoft.Office.Interop.Excel.Workbook OpenDatabase(string Filename, object CommandText, object CommandType, object BackgroundQuery, object ImportDataAs)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(18)]
		[DispId(1924)]
		public void OpenText(string Filename, object Origin, object StartRow, object DataType, Microsoft.Office.Interop.Excel.XlTextQualifier TextQualifier, object ConsecutiveDelimiter, object Tab, object Semicolon, object Comma, object Space, object Other, object OtherChar, object FieldInfo, object TextVisualLayout, object DecimalSeparator, object ThousandsSeparator, object TrailingMinusNumbers, object Local)
		{
			throw new NotImplementedException();
		}

		[DispId(2280)]
		public Microsoft.Office.Interop.Excel.Workbook OpenXML(string Filename, object Stylesheets, object LoadOption)
		{
			throw new NotImplementedException();
		}

		[DispId(150)]
		public dynamic Parent { get { return Application; } }

		#endregion
	}
}
