﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Collections;
using Google.GData.Spreadsheets;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[Guid("91B277AA-9D18-4367-8289-63BEE7FE6EB3")]
	[ComVisible(true)]
	public class Worksheets : Microsoft.Office.Interop.Excel.Sheets
	{
		internal Workbook Workbook { get; private set; }
		internal Worksheet Active { get; set; }
		internal SheetsService Sheets { get; private set; }

		protected List<Worksheet> worksheets;

		internal Worksheets(Workbook workbook)
		{
			Workbook = workbook;
			Sheets = Workbook.Document.Sheets;

			worksheets = new List<Worksheet>();

			foreach (Sheet sheet in Workbook.Document.Sheets.Items)
			{
				worksheets.Add(new Worksheet(this, sheet));
			}

			Active = worksheets.FirstOrDefault();
		}

		#region properties

		/// <summary>
		/// Returns a single object from a collection.
		/// </summary>
		/// <param name="Index"></param>
		/// <returns>Worksheet</returns>
		[DispId(0)]
		public dynamic this[object Index]
		{
			get
			{
				if (Index == null)
				{
					throw new ArgumentNullException("Index");
				}

				if (Index is String)
				{
					foreach (Worksheet worksheet in worksheets)
					{
						if (worksheet.Name.Equals(Index as String))
						{
							return worksheet;
						}
					}

					throw new ArgumentOutOfRangeException("Index");
				}
				else
				{
					return worksheets[(int)Index - 1];
				}
			}
		}

		/// <summary>
		/// Returns an int value that represents the number of objects in the collection.
		/// </summary>
		[DispId(118)]
		public int Count { get { return worksheets.Count; } }

		/// <summary>
		/// Returns a single object from a collection.
		/// </summary>
		/// <param name="Index"></param>
		/// <returns>Worksheet</returns>
		[DispId(170)]
		dynamic Microsoft.Office.Interop.Excel.Sheets.get_Item(object Index)
		{
			return this[Index];
		}

		#endregion

		#region methods

		/// <summary>
		/// Creates a new worksheet. The new worksheet becomes the active sheet.
		/// </summary>
		/// <param name="Before">ignored</param>
		/// <param name="After">ignored</param>
		/// <param name="Count">ignored</param>
		/// <param name="Type">ignored</param>
		/// <returns></returns>
		[DispId(181)]
		public dynamic Add(object Before, object After, object Count, object Type)
		{
			Sheet sheet = Sheets.Add("Sheet" + (Workbook.Document.Sheets.Items.Count + 1));
			var worksheet = new Worksheet(this, sheet);

			worksheets.Add(worksheet);
			Active = worksheet;

			return worksheet;
		}

		internal void Delete(Worksheet worksheet)
		{
			if (worksheets.Contains(worksheet))
			{
				if (worksheets.Count == 1)
				{
					throw new InvalidOperationException("Cannot delete the last sheet");
				}

				Sheets.Delete(worksheet.Sheet);
				worksheets.Remove(worksheet);
				Active = worksheets.First();
			}
		}

		#endregion

		#region unused members

		[DispId(148)]
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get
			{
				return Workbook.Application;
			}
		}

		[DispId(551)]
		public void Copy(object Before, object After)
		{
			throw new NotImplementedException();
		}

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator { get { throw new NotImplementedException(); } }

		[DispId(117)]
		public void Delete()
		{
		    throw new NotImplementedException();
		}

		[DispId(469)]
		public void FillAcrossSheets(Microsoft.Office.Interop.Excel.Range Range, Microsoft.Office.Interop.Excel.XlFillWith Type)
		{
			throw new NotImplementedException();
		}

		[DispId(-4)]
		public IEnumerator GetEnumerator()
		{
			return worksheets.GetEnumerator();
		}

		[DispId(1418)]
		public Microsoft.Office.Interop.Excel.HPageBreaks HPageBreaks { get { throw new NotImplementedException(); } }

		[DispId(637)]
		public void Move(object Before, object After)
		{
			throw new NotImplementedException();
		}

		[DispId(150)]
		public dynamic Parent { get { return Workbook; } }

		[DispId(905)]
		public void _PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate)
		{
			throw new NotImplementedException();
		}

		[DispId(1772)]
		public void PrintOut(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName)
		{
			throw new NotImplementedException();
		}

		[DispId(2361)]
		public void PrintOutEx(object From, object To, object Copies, object Preview, object ActivePrinter, object PrintToFile, object Collate, object PrToFileName, object IgnorePrintAreas)
		{
			throw new NotImplementedException();
		}

		[DispId(281)]
		public void PrintPreview(object EnableChanges)
		{
			throw new NotImplementedException();
		}

		[DispId(235)]
		public void Select(object Replace)
		{
			throw new NotImplementedException();
		}

		[DispId(558)]
		public dynamic Visible { get; set; }

		[DispId(1419)]
		public Microsoft.Office.Interop.Excel.VPageBreaks VPageBreaks { get { throw new NotImplementedException(); } }

		#endregion

		#region excel version specific members
		/**/

		public dynamic Add2([OptionalAttribute]object Before, [OptionalAttribute]object After, [OptionalAttribute]object Count, [OptionalAttribute]object NewLayout)
		{
			throw new NotImplementedException();
		}

		/*//**/
		#endregion
	}
}
